# -*- coding: utf-8 -*-
# FileName: BioGetGenBank
"""
################################################################################################

BioGetGenBank is used to download the DNAs/RNAs, mRNAs, protein sequence from the

GenBank (http://www.ncbi.nlm.nih.gov/Genbank/) website. You can only need input a

GenBank ID (gi) or prepare a file (gilist.txt) related to ID. Then, you can

 obtain a *.fasta file saving sequences you need.  You can freely use

 and distribute it. If you hava  any problem, you could contact with us timely!

Authors: CBDD Group, CSU, China.

Date: 2015.8.17

Email: oriental-cds@163.com

################################################################################################
"""
import urllib2, os

# ########################################################################################
def get_nucleotide_gi_fasta(gi):
    """
    #########################################################################################
    Get the nucleotide fasta sequence from the GenBank website by ID.

    Usage:

    result=get_nucleotide_gi_fasta(gi)

    Input: gi is a string indicating ID such as "392893239           ".

    Output: a file of nucleotide fasta sequence.
    #########################################################################################
    """
    url = 'http://www.ncbi.nlm.nih.gov/sviewer/viewer.cgi?tool=portal&sendto=on&log$=seqview&db=nuccore&dopt=fasta&sort=&val=' + gi + '&from=begin&to=end&maxplex=1'
    try:
        f = urllib2.urlopen(url)
        print 'GI: %s is OK!' % gi
    except:
        print 'GI: %s is not avaliable in GenBank!' % gi
        return False
    with open(os.getcwd() + "/" + gi + '.fasta', "wb") as code:
        code.write(f.read())
    f.close()
    return True


# ########################################################################################
def get_nucleotide_gi_fasta_list(ID_list_file):
    """
    #########################################################################################
    Get the nucleotide fasta sequence from the GenBank website by ID list.

    Usage:

    result=get_nucleotide_gi_fasta_list(ID_list_file)

    Input: ID_list_file is a file indicating IDs such as "392893239".

    Output: several files of nucleotide fasta sequence.
    #########################################################################################
    """
    for i in open(os.getcwd() + "/" + ID_list_file, 'rU'):
        gi = i.strip()
        url = 'http://www.ncbi.nlm.nih.gov/sviewer/viewer.cgi?tool=portal&sendto=on&log$=seqview&db=nuccore&dopt=fasta&sort=&val=' + gi + '&from=begin&to=end&maxplex=1'
        try:
            f = urllib2.urlopen(url)
            print 'GI: %s is OK!' % gi
        except:
            print 'GI: %s is not avaliable in GenBank!' % gi
            return False
        with open(os.getcwd() + "/" + gi + '.fasta', "wb") as code:
            code.write(f.read())
        f.close()
    return True


# ########################################################################################
def get_protein_gi_fasta(gi):
    """
    #########################################################################################
    Get the protein fasta sequence from the GenBank website by ID.

    Usage:

    result=get_protein_gi_fasta(gi)

    Input: gi is a string indicating ID such as "344243512           ".

    Output: a file of protein fasta sequence.
    #########################################################################################
    """
    url = 'http://www.ncbi.nlm.nih.gov/sviewer/viewer.cgi?tool=portal&sendto=on&log$=seqview&db=protein&dopt=fasta&sort=&val=' + gi + '&from=begin&to=end&maxplex=1'
    try:
        f = urllib2.urlopen(url)
        print 'GI: %s is OK!' % gi
    except:
        print 'GI: %s is not avaliable in GenBank!' % gi
        return False
    with open(os.getcwd() + "/" + gi + '.fasta', "wb") as code:
        code.write(f.read())
    f.close()
    return True


# ########################################################################################
def get_protein_gi_fasta_list(ID_list_file):
    """
    #########################################################################################
    Get the protein fasta sequence from the GenBank website by ID list.

    Usage:

    result=get_protein_gi_fasta_list(ID_list_file)

    Input: ID_list_file is a file indicating IDs such as "344243512".

    Output: several files of protein fasta sequence.
    #########################################################################################
    """
    for i in open(os.getcwd() + "/" + ID_list_file, 'rU'):
        gi = i.strip()
        url = 'http://www.ncbi.nlm.nih.gov/sviewer/viewer.cgi?tool=portal&sendto=on&log$=seqview&db=protein&dopt=fasta&sort=&val=' + gi + '&from=begin&to=end&maxplex=1'
        try:
            f = urllib2.urlopen(url)
            print 'GI: %s is OK!' % gi
        except:
            print 'GI: %s is not avaliable in GenBank!' % gi
            return False
        with open(os.getcwd() + "/" + gi + '.fasta', "wb") as code:
            code.write(f.read())
        f.close()
    return True

# ########################################################################################
if __name__ == '__main__':
    get_nucleotide_gi_fasta('392893239')
    print '#' * 20
    
    get_nucleotide_gi_fasta_list('gilist.txt')
    print '#' * 20

    get_protein_gi_fasta('344243512')
    print '#' * 20

    get_protein_gi_fasta_list('gilist_protein.txt')
    print '#' * 20

